@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2001 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: amd.h,v 1.3 1999/02/04 07:24:14 ezk Exp $
+ * $Id: amd.h,v 1.8.2.2 2001/01/12 23:28:55 ro Exp $
  *
  */
 
@@ -106,8 +106,10 @@ struct amu_global_options {
   char *cluster;		/* cluster name */
   char *karch;			/* kernel architecture */
   char *logfile;		/* amd log file */
-  char *op_sys;			/* operating system name */
-  char *op_sys_ver;		/* OS version */
+  char *op_sys;			/* operating system name ${os} */
+  char *op_sys_ver;		/* OS version ${osver} */
+  char *op_sys_full;		/* full OS name ${full_os} */
+  char *op_sys_vendor;		/* name of OS vendor ${vendor} */
   char *pid_file;		/* PID file */
   char *sub_domain;		/* local domain */
   char *map_options;		/* global map options */
@@ -132,6 +134,8 @@ struct amu_global_options {
 #ifdef HAVE_MAP_NIS
   char *nis_domain;		/* YP domain name */
 #endif /* HAVE_MAP_NIS */
+  char *nfs_proto;		/* NFS protocol (NULL, udp, tcp) */
+  int nfs_vers;			/* NFS version (0, 2, 3, 4) */
 };
 
 /* if you add anything here, update conf.c:reset_cf_map() */
@@ -163,6 +167,7 @@ struct mnt_map {
   short flags;                  /* Allocation flags */
   short alloc;                  /* Allocation mode */
   time_t modify;                /* Modify time of map */
+  u_int reloads;		/* Number of times map was reloaded */
   char *map_name;               /* Name of this map */
   char *wildcard;               /* Wildcard value */
   reload_fn *reload;            /* Function to be used for reloads */
@@ -224,7 +229,7 @@ extern voidp amqproc_umnt_1_svc(voidp argp, struct svc_req *rqstp);
 
 /* other external definitions */
 extern am_nfs_fh *root_fh(char *dir);
-extern am_node * autofs_lookuppn(am_node *mp, char *fname, int *error_return, int op);
+extern am_node *autofs_lookuppn(am_node *mp, char *fname, int *error_return, int op);
 extern am_node *find_ap(char *);
 extern am_node *find_ap2(char *, am_node *);
 extern bool_t xdr_amq_mount_info_qelem(XDR *xdrs, qelem *qhead);
@@ -249,8 +254,10 @@ extern void root_newmap(const char *, const char *, const char *, const cf_map_t
 
 /* amd global variables */
 extern FILE *yyin;
-extern SVCXPRT *nfs_program_2_transp;	/* For quick_reply() */
+extern SVCXPRT *nfs_program_2_transp; /* For quick_reply() */
 extern char *conf_tag;
+extern char *opt_gid;
+extern char *opt_uid;
 extern int NumChild;
 extern int fwd_sock;
 extern int select_intr_valid;
@@ -258,20 +265,21 @@ extern int usage;
 extern int use_conf_file;	/* use amd configuration file */
 extern jmp_buf select_intr;
 extern qelem mfhead;
-extern struct amu_global_options gopt;	/* where global options are stored */
+extern struct am_opts fs_static; /* copy of the options to play with */
+extern struct amu_global_options gopt; /* where global options are stored */
 
 #ifdef HAVE_SIGACTION
 extern sigset_t masked_sigs;
 #endif /* HAVE_SIGACTION */
 
-#if defined(HAVE_AM_FS_LINK) || defined(HAVE_AM_FS_LINKX)
+#if defined(HAVE_AMU_FS_LINK) || defined(HAVE_AMU_FS_LINKX)
 extern char *amfs_link_match(am_opts *fo);
 extern int amfs_link_fumount(mntfs *mf);
-#endif /* defined(HAVE_AM_FS_LINK) || defined(HAVE_AM_FS_LINKX) */
+#endif /* defined(HAVE_AMU_FS_LINK) || defined(HAVE_AMU_FS_LINKX) */
 
-#ifdef HAVE_AM_FS_NFSL
+#ifdef HAVE_AMU_FS_NFSL
 extern char *nfs_match(am_opts *fo);
-#endif /* HAVE_AM_FS_NFSL */
+#endif /* HAVE_AMU_FS_NFSL */
 
 #if defined(HAVE_FS_NFS3) && !defined(HAVE_XDR_MOUNTRES3)
 extern bool_t xdr_mountres3(XDR *xdrs, mountres3 *objp);
