@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 Erez Zadok
+ * Copyright (c) 1997-2001 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: amfs_auto.c,v 1.1.1.1 1998/11/05 02:04:46 ezk Exp $
+ * $Id: amfs_auto.c,v 1.9.2.5 2001/01/12 23:28:55 ro Exp $
  *
  */
 
@@ -57,19 +57,17 @@
  ****************************************************************************/
 #define	IN_PROGRESS(cp) ((cp)->mp->am_mnt->mf_flags & MFF_MOUNTING)
 
-/* DEVELOPERS: turn this on for special debugging of readdir code */
-#undef DEBUG_READDIR
+#define DOT_DOT_COOKIE	(u_int) 1
 
 /****************************************************************************
  *** STRUCTURES                                                           ***
  ****************************************************************************/
 
 
-
 /****************************************************************************
  *** FORWARD DEFINITIONS                                                  ***
  ****************************************************************************/
-static int amfs_auto_bgmount(struct continuation * cp, int mpe);
+static int amfs_auto_bgmount(struct continuation *cp, int mpe);
 static int amfs_auto_mount(am_node *mp);
 static int amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, int count, int fully_browsable);
 static void amfs_auto_umounted(am_node *mp);
@@ -448,62 +446,62 @@ try_mount(voidp mvp)
  * Pick a file system to try mounting and
  * do that in the background if necessary
  *
- For each location:
- if it is new -defaults then
- extract and process
- continue;
- fi
- if it is a cut then
- if a location has been tried then
- break;
- fi
- continue;
- fi
- parse mount location
- discard previous mount location if required
- find matching mounted filesystem
- if not applicable then
- this_error = No such file or directory
- continue
- fi
- if the filesystem failed to be mounted then
- this_error = error from filesystem
- elif the filesystem is mounting or unmounting then
- this_error = -1
- elif the fileserver is down then
- this_error = -1
- elif the filesystem is already mounted
- this_error = 0
- break
- fi
- if no error on this mount then
- this_error = initialize mount point
- fi
- if no error on this mount and mount is delayed then
- this_error = -1
- fi
- if this_error < 0 then
- retry = true
- fi
- if no error on this mount then
- make mount point if required
- fi
- if no error on this mount then
- if mount in background then
- run mount in background
- return -1
- else
- this_error = mount in foreground
- fi
- fi
- if an error occurred on this mount then
- update stats
- save error in mount point
- fi
- endfor
+For each location:
+	if it is new -defaults then
+		extract and process
+		continue;
+	fi
+	if it is a cut then
+		if a location has been tried then
+			break;
+		fi
+		continue;
+	fi
+	parse mount location
+	discard previous mount location if required
+	find matching mounted filesystem
+	if not applicable then
+		this_error = No such file or directory
+		continue
+	fi
+	if the filesystem failed to be mounted then
+		this_error = error from filesystem
+	elif the filesystem is mounting or unmounting then
+		this_error = -1
+	elif the fileserver is down then
+		this_error = -1
+	elif the filesystem is already mounted
+		this_error = 0
+		break
+	fi
+	if no error on this mount then
+		this_error = initialize mount point
+	fi
+	if no error on this mount and mount is delayed then
+		this_error = -1
+	fi
+	if this_error < 0 then
+		retry = true
+	fi
+	if no error on this mount then
+		make mount point if required
+	fi
+	if no error on this mount then
+		if mount in background then
+			run mount in background
+			return -1
+		else
+			this_error = mount in foreground
+		fi
+	fi
+	if an error occurred on this mount then
+		update stats
+		save error in mount point
+	fi
+endfor
  */
 static int
-amfs_auto_bgmount(struct continuation * cp, int mpe)
+amfs_auto_bgmount(struct continuation *cp, int mpe)
 {
   mntfs *mf = cp->mp->am_mnt;	/* Current mntfs */
   mntfs *mf_retry = 0;		/* First mntfs which needed retrying */
@@ -579,7 +577,7 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
      * Note whether this is a real mount attempt
      */
     if (p == &amfs_error_ops) {
-      plog(XLOG_MAP, "Map entry %s for %s failed to match", *cp->ivec, mp->am_path);
+      plog(XLOG_MAP, "Map entry %s for %s did not match", *cp->ivec, mp->am_path);
       if (this_error <= 0)
 	this_error = ENOENT;
       continue;
@@ -609,8 +607,8 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
 	 * Don't try logging the string from mf, since it may be bad!
 	 */
 	if (cp->fs_opts.opt_fs != mf->mf_fo->opt_fs)
-	  plog(XLOG_ERROR, "use %s instead of 0x%x",
-	       cp->fs_opts.opt_fs, mf->mf_fo->opt_fs);
+	  plog(XLOG_ERROR, "use %s instead of 0x%lx",
+	       cp->fs_opts.opt_fs, (unsigned long) mf->mf_fo->opt_fs);
 
 	mp->am_link = str3cat((char *) 0,
 			      cp->fs_opts.opt_fs, "/", link_dir);
@@ -670,8 +668,6 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
       else
 	mk_fattr(mp, NFLNK);
 
-      mp->am_fattr.na_fileid = mp->am_gen;
-
       if (p->fs_init)
 	this_error = (*p->fs_init) (mf);
     }
@@ -695,7 +691,7 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
       int i = atoi(mf->mf_fo->opt_delay);
       if (i > 0 && clocktime() < (cp->start + i)) {
 #ifdef DEBUG
-	dlog("Mount of %s delayed by %ds", mf->mf_mount, i - clocktime() + cp->start);
+	dlog("Mount of %s delayed by %lds", mf->mf_mount, (long) (i - clocktime() + cp->start));
 #endif /* DEBUG */
 	this_error = -1;
       }
@@ -705,6 +701,10 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
       if (!mf_retry)
 	mf_retry = dup_mntfs(mf);
       cp->retry = TRUE;
+#ifdef DEBUG
+      dlog("will retry ...\n");
+#endif /* DEBUG */
+      break;
     }
 
     if (!this_error) {
@@ -717,7 +717,10 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
 	  untimeout(cp->callout);
 	  cp->callout = 0;
 	}
+
+	/* actually run the task, backgrounding as necessary */
 	run_task(try_mount, (voidp) mp, amfs_auto_cont, (voidp) cp);
+
 	mf->mf_flags |= MFF_MKMNT;	/* XXX */
 	if (mf_retry)
 	  free_mntfs(mf_retry);
@@ -764,8 +767,9 @@ amfs_auto_bgmount(struct continuation * cp, int mpe)
      * Rewind the location vector and
      * reset the default options.
      */
-    cp->ivec = cp->xivec;
-    cp->def_opts = strealloc(cp->def_opts, cp->auto_opts);
+#ifdef DEBUG
+    dlog("(skipping rewind)\n");
+#endif /* DEBUG */
     /*
      * Arrange that amfs_auto_bgmount is called
      * after anything else happens.
@@ -992,7 +996,7 @@ amfs_auto_lookuppn(am_node *mp, char *fname, int *error_return, int op)
   if (error) {
 #ifdef DEBUG
     errno = error;		/* XXX */
-    dlog("Returning error: %m", error);
+    dlog("Returning error: %m");
 #endif /* DEBUG */
     XFREE(fname);
     ereturn(error);
@@ -1126,11 +1130,18 @@ amfs_auto_lookuppn(am_node *mp, char *fname, int *error_return, int op)
 	char **sp = rvec;
 	while (*sp) {		/* loop until you find something, if any */
 	  memset((char *) &ap, 0, sizeof(am_opts));
+	  /*
+	   * This next routine cause many spurious "expansion of ... is"
+	   * messages, which are ignored, b/c all we need out of this
+	   * routine is to match selectors.  These spurious messages may
+	   * be wrong, esp. if they try to expand ${key} b/c it will
+	   * get expanded to "/defaults"
+	   */
 	  pt = ops_match(&ap, *sp, "", mp->am_path, "/defaults",
 			 mp->am_parent->am_mnt->mf_info);
 	  free_opts(&ap);	/* don't leak */
 	  if (pt == &amfs_error_ops) {
-	    plog(XLOG_MAP, "failed to match defaults for \"%s\"", *sp);
+	    plog(XLOG_MAP, "did not match defaults for \"%s\"", *sp);
 	  } else {
 	    dfl = strip_selectors(*sp, "/defaults");
 	    plog(XLOG_MAP, "matched default selectors \"%s\"", dfl);
@@ -1299,6 +1310,10 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
   u_int gen = *(u_int *) cookie;
   am_node *xp;
   mntent_t mnt;
+#ifdef DEBUG
+  nfsentry *ne;
+  static int j;
+#endif /* DEBUG */
 
   dp->dl_eof = FALSE;		/* assume readdir not done */
 
@@ -1311,6 +1326,7 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
       return amfs_auto_readdir_browsable(mp, cookie, dp, ep, count, FALSE);
   }
 
+  /* when gen is 0, we start reading from the beginning of the directory */
   if (gen == 0) {
     /*
      * In the default instance (which is used to start a search) we return
@@ -1321,7 +1337,7 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
      * fairly unbelievable) then tough.
      */
 #ifdef DEBUG
-    dlog("default search");
+    dlog("amfs_auto_readdir: default search");
 #endif /* DEBUG */
     /*
      * Check for enough room.  This is extremely approximate but is more
@@ -1350,24 +1366,33 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
       ep[1].ne_fileid = mp->am_gen;
     ep[1].ne_name = "..";
     ep[1].ne_nextentry = 0;
-    *(u_int *) ep[1].ne_cookie =
-      xp ? xp->am_gen : ~(u_int) 0;
+    *(u_int *) ep[1].ne_cookie = (xp ? xp->am_gen : DOT_DOT_COOKIE);
 
     if (!xp)
       dp->dl_eof = TRUE;	/* by default assume readdir done */
 
+#ifdef DEBUG
+    amuDebug(D_READDIR)
+      for (j=0,ne=ep; ne; ne=ne->ne_nextentry)
+	plog(XLOG_DEBUG, "gen1 key %4d \"%s\" fi=%d ck=%d",
+	     j++, ne->ne_name, ne->ne_fileid, *(u_int *)ne->ne_cookie);
+#endif /* DEBUG */
     return 0;
   }
 #ifdef DEBUG
-  dlog("real child");
+  dlog("amfs_auto_readdir: real child");
 #endif /* DEBUG */
 
-  if (gen == ~(u_int) 0) {
+  if (gen == DOT_DOT_COOKIE) {
 #ifdef DEBUG
-    dlog("End of readdir in %s", mp->am_path);
+    dlog("amfs_auto_readdir: End of readdir in %s", mp->am_path);
 #endif /* DEBUG */
     dp->dl_eof = TRUE;
     dp->dl_entries = 0;
+#ifdef DEBUG
+    amuDebug(D_READDIR)
+      plog(XLOG_DEBUG, "end of readdir eof=TRUE, dl_entries=0\n");
+#endif /* DEBUG */
     return 0;
   }
 
@@ -1379,6 +1404,7 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
   if (xp) {
     int nbytes = count / 2;	/* conservative */
     int todo = MAX_READDIR_ENTRIES;
+
     dp->dl_entries = ep;
     do {
       am_node *xp_next = next_nonerror_node(xp->am_osib);
@@ -1386,7 +1412,7 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
       if (xp_next) {
 	*(u_int *) ep->ne_cookie = xp_next->am_gen;
       } else {
-	*(u_int *) ep->ne_cookie = ~(u_int) 0;
+	*(u_int *) ep->ne_cookie = DOT_DOT_COOKIE;
 	dp->dl_eof = TRUE;
       }
 
@@ -1409,6 +1435,12 @@ amfs_auto_readdir(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsentry *ep, i
 
     ep->ne_nextentry = 0;
 
+#ifdef DEBUG
+    amuDebug(D_READDIR)
+      for (j=0,ne=ep; ne; ne=ne->ne_nextentry)
+	plog(XLOG_DEBUG, "gen2 key %4d \"%s\" fi=%d ck=%d",
+	     j++, ne->ne_name, ne->ne_fileid, *(u_int *)ne->ne_cookie);
+#endif /* DEBUG */
     return 0;
   }
   return ESTALE;
@@ -1422,17 +1454,18 @@ amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsen
   u_int gen = *(u_int *) cookie;
   int chain_length, i;
   static nfsentry *te, *te_next;
-#ifdef DEBUG_READDIR
+#ifdef DEBUG
   nfsentry *ne;
   static int j;
-#endif /* DEBUG_READDIR */
+#endif /* DEBUG */
 
   dp->dl_eof = FALSE;		/* assume readdir not done */
 
-#ifdef DEBUG_READDIR
-  plog(XLOG_INFO, "amfs_auto_readdir_browsable gen=%u, count=%d",
-       gen, count);
-#endif /* DEBUG_READDIR */
+#ifdef DEBUG
+  amuDebug(D_READDIR)
+    plog(XLOG_DEBUG, "amfs_auto_readdir_browsable gen=%u, count=%d",
+	 gen, count);
+#endif /* DEBUG */
 
   if (gen == 0) {
     /*
@@ -1444,7 +1477,7 @@ amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsen
      * fairly unbelievable) then tough.
      */
 #ifdef DEBUG
-    dlog("default search");
+    dlog("amfs_auto_readdir_browsable: default search");
 #endif /* DEBUG */
     /*
      * Check for enough room.  This is extremely approximate but is more
@@ -1482,9 +1515,10 @@ amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsen
       ep[1].ne_fileid = mp->am_parent->am_gen;
     else
       ep[1].ne_fileid = mp->am_gen;
+
     ep[1].ne_name = "..";
     ep[1].ne_nextentry = 0;
-    *(u_int *) ep[1].ne_cookie = ~(u_int) 0;
+    *(u_int *) ep[1].ne_cookie = DOT_DOT_COOKIE;
 
     /*
      * If map is browsable, call a function make_entry_chain() to construct
@@ -1495,11 +1529,11 @@ amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsen
     te = make_entry_chain(mp, dp->dl_entries, fully_browsable);
     if (!te)
       return 0;
-#ifdef DEBUG_READDIR
-    j = 0;
-    for (ne=te; ne; ne=ne->ne_nextentry)
-      plog(XLOG_INFO, "gen1 key %4d \"%s\"", j++, ne->ne_name);
-#endif /* DEBUG_READDIR */
+#ifdef DEBUG
+    amuDebug(D_READDIR)
+      for (j=0,ne=te; ne; ne=ne->ne_nextentry)
+	plog(XLOG_DEBUG, "gen1 key %4d \"%s\"", j++, ne->ne_name);
+#endif /* DEBUG */
 
     /* return only "chain_length" entries */
     te_next = te;
@@ -1517,20 +1551,26 @@ amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsen
       dp->dl_eof = TRUE;	/* tell readdir that's it */
     }
     ep[1].ne_nextentry = te;	/* append this chunk of "te" chain */
-#ifdef DEBUG_READDIR
-    for (ne=te; ne; ne=ne->ne_nextentry)
-      plog(XLOG_INFO, "gen2 key %4d \"%s\"", j++, ne->ne_name);
+#ifdef DEBUG
+    amuDebug(D_READDIR) {
+      for (j=0,ne=te; ne; ne=ne->ne_nextentry)
+	plog(XLOG_DEBUG, "gen2 key %4d \"%s\"", j++, ne->ne_name);
+      for (j=0,ne=ep; ne; ne=ne->ne_nextentry)
+	plog(XLOG_DEBUG, "gen2+ key %4d \"%s\" fi=%d ck=%d",
+	     j++, ne->ne_name, ne->ne_fileid, *(u_int *)ne->ne_cookie);
+      plog(XLOG_DEBUG, "EOF is %d", dp->dl_eof);
+    }
 #endif /* DEBUG_READDIR */
     return 0;
   } /* end of "if (gen == 0)" statement */
 
 #ifdef DEBUG
-  dlog("real child");
+  dlog("amfs_auto_readdir_browsable: real child");
 #endif /* DEBUG */
 
-  if (gen == ~(u_int) 0) {
+  if (gen == DOT_DOT_COOKIE) {
 #ifdef DEBUG
-    dlog("End of readdir in %s", mp->am_path);
+    dlog("amfs_auto_readdir_browsable: End of readdir in %s", mp->am_path);
 #endif /* DEBUG */
     dp->dl_eof = TRUE;
     dp->dl_entries = 0;
@@ -1570,12 +1610,14 @@ amfs_auto_readdir_browsable(am_node *mp, nfscookie cookie, nfsdirlist *dp, nfsen
   }
   ep = te;			/* send next chunk of "te" chain */
   dp->dl_entries = ep;
-#ifdef DEBUG_READDIR
-  plog(XLOG_INFO, "dl_entries=0x%x, te_next=0x%x, dl_eof=%d",
-       dp->dl_entries, te_next, dp->dl_eof);
-  for (ne=te; ne; ne=ne->ne_nextentry)
-    plog(XLOG_INFO, "gen3 key %4d \"%s\"", j++, ne->ne_name);
-#endif /* DEBUG_READDIR */
+#ifdef DEBUG
+  amuDebug(D_READDIR) {
+    plog(XLOG_DEBUG, "dl_entries=0x%lx, te_next=0x%lx, dl_eof=%d",
+	 (long) dp->dl_entries, (long) te_next, dp->dl_eof);
+    for (ne=te; ne; ne=ne->ne_nextentry)
+      plog(XLOG_DEBUG, "gen3 key %4d \"%s\"", j++, ne->ne_name);
+  }
+#endif /* DEBUG */
   return 0;
 }
 
