@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2001 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: mount_linux.c,v 1.5 1999/08/16 01:16:30 ezk Exp $
+ * $Id: mount_linux.c,v 1.11.2.4 2001/02/01 21:17:33 ib42 Exp $
  */
 
 /*
@@ -154,12 +154,12 @@ parse_opts(char *type, char *optstr, int *flags, char **xopts, int *noauto)
        * and parse the fs-specific options
        */
 #ifdef MOUNT_TYPE_PCFS
-      if (!STREQ(type, MOUNT_TYPE_PCFS))
+      if (STREQ(type, MOUNT_TYPE_PCFS))
 	dev_opts = dos_opts;
       else
 #endif /* MOUNT_TYPE_PCFS */
 #ifdef MOUNT_TYPE_CDFS
-	if (!STREQ(type, MOUNT_TYPE_CDFS))
+	if (STREQ(type, MOUNT_TYPE_CDFS))
 	  dev_opts = iso_opts;
 	else
 #endif /* MOUNT_TYPE_CDFS */
@@ -217,7 +217,6 @@ mount_linux(MTYPE_TYPE type, mntent_t *mnt, int flags, caddr_t data)
   int noauto = 0;
   int errorcode;
   nfs_args_t *mnt_data = (nfs_args_t *) data;
-  int nfs_def_file_io_buffer_size = 1024;
 
   if (mnt->mnt_opts && STREQ (mnt->mnt_opts, "defaults"))
     mnt->mnt_opts = NULL;
@@ -247,16 +246,18 @@ mount_linux(MTYPE_TYPE type, mntent_t *mnt, int flags, caddr_t data)
 #endif /* MNT2_NFS_OPT_NOAC */
 
     /*
-     * Linux kernels 2.0.x and earlier used a default NFS read/write size of
-     * 1024 bytes.  2.1 kernels and newer use a 4KB rsize/wsize.
-     * NOTE: 131326 => linux-2.1.0
+     * in nfs structure implementation version 4, the old
+     * filehandle field was renamed "old_root" and left as 3rd field,
+     * while a new field called "root" was added to the end of the
+     * structure.
      */
-    if (linux_version_code() >= 131326)
-      nfs_def_file_io_buffer_size = 4096;
-    if (!mnt_data->rsize)
-      mnt_data->rsize = nfs_def_file_io_buffer_size;
-    if (!mnt_data->wsize)
-      mnt_data->wsize = nfs_def_file_io_buffer_size;
+#ifdef MNT2_NFS_OPT_VER3
+    if (mnt_data->flags & MNT2_NFS_OPT_VER3)
+      memset(mnt_data->old_root.data, 0, FHSIZE);
+    else
+#endif /* MNT2_NFS_OPT_VER3 */
+      memcpy(mnt_data->old_root.data, mnt_data->root.data, FHSIZE);
+
 #ifdef HAVE_FIELD_NFS_ARGS_T_BSIZE
     /* linux mount version 3 */
     mnt_data->bsize = 0;	/* let the kernel decide */
@@ -281,9 +282,8 @@ mount_linux(MTYPE_TYPE type, mntent_t *mnt, int flags, caddr_t data)
     /*
      * connect() the socket for kernels 1.3.10 and below
      * only to avoid problems with multihomed hosts.
-     * NOTE: 66314 => linux-1.3.10
      */
-    if (linux_version_code() <= 66314) {
+    if (linux_version_code() <= 0x01030a) {
       int ret = connect(mnt_data->fd,
 			(struct sockaddr *) &mnt_data->addr,
 			sizeof(mnt_data->addr));
@@ -342,8 +342,11 @@ mount_linux(MTYPE_TYPE type, mntent_t *mnt, int flags, caddr_t data)
     plog(XLOG_DEBUG, "linux mount: dir %s\n", mnt->mnt_dir);
   }
   amuDebug(D_TRACE) {
-    plog(XLOG_DEBUG, "linux mount: updated nfs_args...");
-    print_nfs_args(mnt_data, 0);
+    plog(XLOG_DEBUG, "linux mount: Generic mount flags 0x%x", MS_MGC_VAL | flags);
+    if (STREQ(type, MOUNT_TYPE_NFS)) {
+      plog(XLOG_DEBUG, "linux mount: updated nfs_args...");
+      print_nfs_args(mnt_data, 0);
+    }
   }
 #endif /* DEBUG */
 
@@ -394,8 +397,6 @@ fail:
  * other architectures yet.
  */
 
-#define nfs_error	linux_nfs_error
-
 #define NE_PERM		1
 #define NE_NOENT	2
 #define NE_IO		5
@@ -418,8 +419,12 @@ fail:
 #define NFS_LOMAP	2
 #define NFS_HIMAP	123
 
+/*
+ * The errno's below are correct for Linux/i386. One day, somebody
+ * with lots of energy ought to verify them against the other ports...
+ */
 static int nfs_errormap[] = {
-	0,		/* (2, unused)		*/
+	0,		/* unused (2)		*/
 	NE_INVAL,	/* ESRCH (3)		*/
 	NE_IO,		/* EINTR (4)		*/
 	NE_IO,		/* EIO (5)		*/
@@ -458,6 +463,7 @@ static int nfs_errormap[] = {
 	NE_INVAL,	/* ENOSYS (38)		*/
 	NE_NOTEMPTY,	/* ENOTEMPTY (39)	*/
 	NE_INVAL,	/* ELOOP (40)		*/
+	NE_INVAL,	/* unused (41)		*/
 	NE_INVAL,	/* ENOMSG (42)		*/
 	NE_INVAL,	/* EIDRM (43)		*/
 	NE_INVAL,	/* ECHRNG (44)		*/
@@ -474,7 +480,7 @@ static int nfs_errormap[] = {
 	NE_INVAL,	/* ENOANO (55)		*/
 	NE_INVAL,	/* EBADRQC (56)		*/
 	NE_INVAL,	/* EBADSLT (57)		*/
-	NE_INVAL,	/* 58: unused		*/
+	NE_INVAL,	/* unused (58)		*/
 	NE_INVAL,	/* EBFONT (59)		*/
 	NE_INVAL,	/* ENOSTR (60)		*/
 	NE_INVAL,	/* ENODATA (61)		*/
