@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 Erez Zadok
+ * Copyright (c) 1997-2001 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: mtab_svr4.c,v 1.1.1.1 1998/11/05 02:04:37 ezk Exp $
+ * $Id: mtab_svr4.c,v 1.4.2.2 2001/01/10 03:23:19 ezk Exp $
  *
  * How to manage the mount table file.  Based on other SVR3 ports.
  *      -Erez Zadok <ezk@cs.columbia.edu>
@@ -58,13 +58,10 @@
  */
 static int mntent_lock_fd = -1;
 
-/* static char mnttabname[] = MNTTAB_FILE_NAME; */
 
-#ifdef MNTTAB_LOCK_FILE
-static char mtlckname[] = MNTTAB_LOCK_FILE;
-#else /* not MTAB_LOCK_FILE */
+#ifdef MOUNT_TABLE_ON_FILE
 static char mtlckname[] = "/etc/.mnttab.lock";
-#endif /* not MTAB_LOCK_FILE */
+#endif /* MOUNT_TABLE_ON_FILE */
 
 
 /****************************************************************************/
@@ -74,13 +71,16 @@ static char mtlckname[] = "/etc/.mnttab.lock";
 static void
 unlockmnttab(void)
 {
+#ifdef MOUNT_TABLE_ON_FILE
   if (mntent_lock_fd >= 0) {
     close(mntent_lock_fd);
     mntent_lock_fd = -1;
   }
+#endif /* MOUNT_TABLE_ON_FILE */
 }
 
 
+#ifdef MOUNT_TABLE_ON_FILE
 static int
 lockfile(int fd, int type)
 {
@@ -99,12 +99,14 @@ lockfile(int fd, int type)
   ret = fcntl(fd, F_SETLKW, &lk);
   return ret;
 }
+#endif /* MOUNT_TABLE_ON_FILE */
+
 
 /* return 0 if locking succeeded, -1 if failed */
 static int
 lockmnttab(void)
 {
-
+#ifdef MOUNT_TABLE_ON_FILE
   /* if mnttab file is locked, all is well */
   if (mntent_lock_fd >= 0)
     return 0;
@@ -125,6 +127,9 @@ lockmnttab(void)
 #endif /* DEBUG */
     return -1;
   }
+#else /* not MOUNT_TABLE_ON_FILE */
+  /* fake lock for in-kernel mount table */
+#endif /* not MOUNT_TABLE_ON_FILE */
 
   /* finally, succeeded in also locking the file */
   return 0;
@@ -154,6 +159,7 @@ mnt_dup(const mntent_t *mtp)
 /*
  * Adjust arguments in mntent_t.
  */
+#ifdef MOUNT_TABLE_ON_FILE
 static mntent_t *
 update_mnttab_fields(const mntent_t *mnt)
 {
@@ -182,12 +188,15 @@ update_mnttab_fields(const mntent_t *mnt)
 
   return &mt;
 }
+#endif /* MOUNT_TABLE_ON_FILE */
 
 
 static void
 write_mntent_to_mtab(FILE *fp, const mntent_t *mnt)
 {
+#ifdef MOUNT_TABLE_ON_FILE
   putmntent(fp, update_mnttab_fields(mnt));
+#endif /* MOUNT_TABLE_ON_FILE */
 }
 
 
